# Generated by Django 3.2.19 on 2023-06-09 13:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0029_enterpriseoffer'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='offer_id',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriseoffer',
            name='offer_id',
            field=models.CharField(max_length=255, primary_key=True, serialize=False),
        ),
    ]
