# Generated by Django 1.11.15 on 2018-09-07 17:57


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0014_enterpriseuser_created'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='enterpriseuser',
            options={'ordering': ['-user_email'], 'verbose_name': 'Enterprise User', 'verbose_name_plural': 'Enterprise Users'},
        ),
    ]
