# Generated by Django 2.2.17 on 2021-07-03 18:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0024_auto_20210602_1811'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='enterpriselearner',
            options={'ordering': ['-user_email'], 'verbose_name': 'Enterprise Learner', 'verbose_name_plural': 'Enterprise Learner'},
        ),
    ]
